<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @package    Zlick Pzyments
 * @subpackage Zlick Pzyments/admin
 * @author     Saad Waseem<saad.waseem@shahruh.com>
 */
class Zlick_Payments_Admin {
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
     * Holds the values to be used in the fields callbacks
     */
    private $options = [];

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		add_action( 'admin_menu', [ $this, 'zlick_payments_config' ] );
		add_action( 'admin_init', [ $this, 'zlick_settings_init' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_styles' ] );
		add_action( 'add_meta_boxes', [ $this, 'zp_add_custom_fields_meta_box' ] );
		add_action( 'save_post', [ $this, 'zp_save_zlick_fields_meta' ] );
	}
	
   	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_register_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/zlick-payments-admin.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->plugin_name );
	}
	/**
	 * Adds data on admin menu page.
	*
	* @return void
	*/
	function zlick_payments_config() {
		add_options_page(
			__( 'Zlick Payments Configuration', $this->plugin_name ),
			__( 'Zlick Payments', $this->plugin_name ),
			'manage_options',
			'zlick-payments-plugin',
			array( $this, 'zlick_payments_page' )
		);
	}

	/**
	 * setup zlick payment options
	 * 
	 * @return void
	 */
	function zlick_settings_init() { // whitelist options
		//register options
		register_setting( 'zlick-payments', 'zlick_payments_settings', array( $this, 'zp_validate_options' ));

		//register zlick payments admin section
		add_settings_section(
			'zlick_admin_settings_section',
			__( 'Zlick Payments Configuration', $this->plugin_name ),
			array( $this, 'print_section_info' ),
			'zlick-payments-plugin'
		);

		// Client Token
		add_settings_field(
            'zp_client_token',
            __( 'Publisher ID', 'zlick-payments' ),
            array( $this, 'field_client_token_render' ),
            'zlick-payments-plugin',
            'zlick_admin_settings_section'         
        );      
		// Client Secret
		add_settings_field(
            'zp_client_secret',
            __( 'API Key', 'zlick-payments' ),
            array( $this, 'field_client_secret_render' ),
            'zlick-payments-plugin',
            'zlick_admin_settings_section'         
        );      
		// Previewable paragraphs
		add_settings_field(
            'zp_previewable_paras',
            __( 'Preview Paragraphs', 'zlick-payments' ),
            array( $this, 'field_previewable_paras_render' ),
            'zlick-payments-plugin',
            'zlick_admin_settings_section'         
        );      

	}

	/**
     * Options page callback
     */
    public function zlick_payments_page()
    {
        // Set class / check user capabilities
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}
	
		if ( isset( $_GET['zlick-payments-plugin'] ) ) {
			// add settings saved message with the class of "updated"
			add_settings_error( 'zlick_payments_settings', 'zlick_payments', __( 'Settings Saved', $this->plugin_name ), 'updated' );
		}
	
		// show error/update messages
		settings_errors( 'zlick_payments_settings' );
		$saved_options = get_option( 'zlick_payments_settings' );

		if( empty($saved_options) ) { //Before First Installation
			$this->options = array(
				'zp_client_token' => '',
				'zp_client_secret' => '',
				'zp_previewable_paras' => '',
			);
		}else{
			$this->options = $saved_options;
		}
	?>
	<div class="wrap">
		<h1><?php __( 'Zlick Payments Configuration', $this->plugin_name ) ?></h1>
		<form method="post" action="options.php">
			<?php
                // This prints out all hidden setting fields
                settings_fields( 'zlick-payments' );
                do_settings_sections( 'zlick-payments-plugin' );
				submit_button();
			?>
        </form>
    </div>
	<?php
    }
	
	/**
	 * Verifies Zlick account.
	 *
	 * @param int $client_token The shop id.
	 * @param string $shop_secret The shop password.
	 *
	 * @return array
	 */
	function zp_authenticate( $client_token, $shop_secret ) {
		$query_string  = "auth={$client_token}|{$shop_secret}&version=cust-1.0.0&type=request&charset=iso";
		$url           = URL_GET_SETTINGS . '?' . $query_string;
		$response      = wp_remote_get( $url );
		print_r($response); exit;
		if( is_array( $response ) && ! is_wp_error( $response ) ){
			$server_output = wp_remote_retrieve_body( $response );
			if ( 'Access denied' === $server_output ) {
				$response = array(
					'status'  => 'error',
					'message' => __( 'Shop ID & Password Required', 'zlick-payments' ),
				);
			} else {
				$response = array(
					'status'  => 'success',
					'message' => __( 'Settings Updated', 'zlick-payments' ),
				);
			}
		}
		return $response;
	}

	/**
     * Sanitize each setting field as needed
     *
     * @param array $post_data Contains all settings fields as array keys
     */
    public function zp_validate_options( $post_data )
    {
		$configurations = array(
			'zp_client_token' => '',
			'zp_client_secret' => '',
			'zp_previewable_paras' => ZLICK_PREVIEWABLE_CONTENT_PARA_DEFAULT,
		);

        if( isset( $post_data['zp_client_token'] ) ){
			$configurations['zp_client_token'] = sanitize_text_field( wp_unslash( $post_data['zp_client_token'] ) );
		}

        if( isset( $post_data['zp_client_secret'] ) ){
			$configurations['zp_client_secret'] = sanitize_text_field( wp_unslash( $post_data['zp_client_secret'] ) );
		}

		if( isset( $post_data['zp_previewable_paras'] ) ){
			$configurations['zp_previewable_paras'] =  sanitize_text_field( $post_data['zp_previewable_paras'] );
		}
		
		return $configurations;
    }

    /** 
     * Print the Section text
     */
    public function print_section_info()
    {
        print __( 'Zlick Payments Configuration', $this->plugin_name );
    }

    /** 
     * Render Client token field markup
     */
    public function field_client_token_render()
    {
		printf(
			'<input type="text" name="zlick_payments_settings[zp_client_token]" id="zp_client_token" value="%s" size="30">', 
			$this->options['zp_client_token']
		);
    }

    /** 
     * Render client secret field markup
     */
    public function field_client_secret_render()
    {
		printf(
			'<input type="text" name="zlick_payments_settings[zp_client_secret]" id="zp_client_secret" value="%s" size="30">', 
			$this->options['zp_client_secret']
		);
	}

    /** 
     * Render previewable paras field markup
     */
    public function field_previewable_paras_render()
    {
		printf(
			'<input type="number" name="zlick_payments_settings[zp_previewable_paras]" id="zp_previewable_paras" value="%s" size="10">', 
			$this->options['zp_previewable_paras']
		);
	}
	
	/**
     * @param $post_id
     *
     * @return mixed
     */
    function zp_save_zlick_fields_meta( $post_id ) {
        // verify nonce
        if ( isset($_POST['zp_meta_box_nonce']) && !wp_verify_nonce( $_POST['zp_meta_box_nonce'], basename(__FILE__) ) ) {
            return $post_id;
        }
        // check autosave
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return $post_id;
        }
        // check permissions
        if ( get_post_type( $post_id ) === ZLICK_POST_TYPE_POST ) {
            isset( $_POST['zp_is_paid'] ) ? update_post_meta( $post_id, 'zp_is_paid', sanitize_text_field( $_POST['zp_is_paid']) ) : '';
            isset( $_POST['zp_article_name'] ) ? update_post_meta( $post_id, 'zp_article_name', sanitize_text_field( $_POST['zp_article_name'] ) ) : '';
        }
    }

    /**
     * Adds Zlick Payments custom fields.
     */
    function zp_add_custom_fields_meta_box() {
        add_meta_box(
            'zlick_payments_fields',
            'Zlick Payment Fields',
            array( $this, 'zp_show_custom_fields_meta_box' ),
            ZLICK_POST_TYPE_POST,
            'normal',
            'high'
        );
	}

	/**
	 * Show Zlick Custom Fields.
	 */
	function zp_show_custom_fields_meta_box() {
		global $post;
		$custom = get_post_custom( $post->ID );
	?>
		<input type="hidden" name="zp_meta_box_nonce" value="<?php echo wp_create_nonce( basename(__FILE__) ); ?>">
		<p style="display:none;">
			Placeholder code: <strong>zp_placeholder</strong>
		</p>
		<p>
			<label for="zp_is_paid">Is Paid</label>
			<input name="zp_is_paid" type="checkbox" id="zp_is_paid" value="paid" <?php echo ( @$custom["zp_is_paid"][0] =="paid") ? "checked" : ""?>   >
		</p>
		<p>
			<label for="zp_article_name">Article Friendly Name</label>
			<input name="zp_article_name" type="text" id="zp_article_name" value="<?php echo @$custom["zp_article_name"][0] ?>">
		</p>
	<?php 
	}
}

if ( is_admin() ) {
	$zlick_payments_admin = new Zlick_Payments_Admin( ZLICK_PLUGIN_NAME, ZLICK_PLUGIN_VERSION );
}